import { computed, shallowRef, watchEffect, createVNode, resolveComponent, mergeProps } from 'vue';

function useSelection(rowSelectionRef, configRef) {
  const {
    getRowKey,
    pageData
  } = configRef;
  const mergedRowSelection = computed(() => {
    var _a;
    const tmp = (_a = rowSelectionRef.value) != null ? _a : {};
    return {
      type: "checkbox",
      ...tmp
    };
  });
  const mergedSelectedKeys = computed(() => {
    return mergedRowSelection.value.selectedRowKeys;
  });
  const selectKeysState = computed(() => {
    return mergedSelectedKeys.value;
  });
  const derivedSelectedKey = computed(() => selectKeysState.value);
  const derivedSelectedKeySet = computed(() => {
    const keys = derivedSelectedKey.value;
    return new Set(Array.isArray(keys) ? keys : [keys]);
  });
  const preserveRecords = shallowRef(new Map());
  const rowKeys = computed(() => {
    return pageData.value.map((item) => getRowKey(item)).filter((key) => !isCheckboxDisabled(key));
  });
  const checkboxPropsMap = computed(() => {
    const getCheckboxProps = mergedRowSelection.value.getCheckboxProps;
    const map = new Map();
    pageData.value.forEach((row) => {
      const key = getRowKey(row);
      const checkboxProps = (getCheckboxProps == null ? void 0 : getCheckboxProps(row)) || {};
      map.set(key, checkboxProps);
    });
    return map;
  });
  const isCheckboxDisabled = (key) => {
    var _a;
    return !!((_a = checkboxPropsMap.value.get(key)) == null ? void 0 : _a.disabled);
  };
  watchEffect(() => {
    const keys = mergedSelectedKeys.value;
    updateRecordsCache(Array.isArray(keys) ? keys : [keys]);
  });
  function updateRecordsCache(keys) {
    if (mergedRowSelection.value.preserveRowKeys) {
      const newCache = new Map();
      keys.forEach((key) => {
        let record = configRef.getRecordByKey(key);
        if (!record && preserveRecords.value.has(key)) {
          record = preserveRecords.value.get(key);
        }
        newCache.set(key, record);
      });
      preserveRecords.value = newCache;
    }
  }
  function setSelectedKeys(keys, record) {
    let avaliableKeys = [];
    let records = [];
    updateRecordsCache(keys);
    const {
      preserveRowKeys,
      onChange: onSelectionChange
    } = mergedRowSelection.value;
    if (preserveRowKeys) {
      avaliableKeys = keys;
      records = keys.map((key) => preserveRecords.value.get(key));
    } else {
      keys.forEach((key) => {
        const record2 = configRef.getRecordByKey(key);
        if (record2 !== void 0) {
          avaliableKeys.push(key);
          records.push(record2);
        }
      });
    }
    onSelectionChange == null ? void 0 : onSelectionChange(avaliableKeys, records, record);
  }
  function setSelectedKey(key, record) {
    const {
      onChange: onSelectionChange
    } = mergedRowSelection.value;
    onSelectionChange == null ? void 0 : onSelectionChange(key, record);
  }
  function renderCell({
    row
  }, _config) {
    return mergedRowSelection.value.type === "radio" ? renderRadio(row) : renderCheckbox(row);
  }
  function renderRadio(row) {
    var _a, _b;
    const keySelected = derivedSelectedKey.value;
    const key = getRowKey(row);
    const options = ((_b = (_a = mergedRowSelection.value).getCheckboxProps) == null ? void 0 : _b.call(_a, row)) || {};
    function onRowSelect() {
      setSelectedKey(key, row);
    }
    return createVNode(resolveComponent("el-radio"), mergeProps(options, {
      "label": true,
      "model-value": key === keySelected,
      "onClick": (e) => e.stopPropagation(),
      "onChange": () => onRowSelect()
    }), {
      default: () => [""]
    });
  }
  function renderCheckbox(row) {
    var _a, _b;
    const keySet = new Set(derivedSelectedKeySet.value);
    const key = getRowKey(row);
    const options = ((_b = (_a = mergedRowSelection.value).getCheckboxProps) == null ? void 0 : _b.call(_a, row)) || {};
    function onRowSelect() {
      const rowkey = getRowKey(row);
      if (keySet.has(rowkey)) {
        keySet.delete(rowkey);
      } else {
        keySet.add(rowkey);
      }
      setSelectedKeys(Array.from(keySet), row);
    }
    return createVNode(resolveComponent("el-checkbox"), mergeProps(options, {
      "model-value": keySet.has(key),
      "onClick": (e) => e.stopPropagation(),
      "onChange": () => onRowSelect()
    }), null);
  }
  function renderTop() {
    const keySet = new Set(derivedSelectedKeySet.value);
    const disabledChecked = computed(() => rowKeys.value.length === 0);
    const checkedCurrentAll = computed(() => rowKeys.value.every((key) => keySet.has(key)));
    const checkedCurrentSome = computed(() => rowKeys.value.some((key) => keySet.has(key)));
    function onSelectAllChange() {
      if (checkedCurrentAll.value) {
        rowKeys.value.forEach((key) => {
          keySet.delete(key);
        });
      } else {
        rowKeys.value.forEach((key) => {
          if (!keySet.has(key)) {
            keySet.add(key);
          }
        });
      }
      setSelectedKeys(Array.from(keySet));
    }
    return mergedRowSelection.value.type === "radio" ? null : createVNode(resolveComponent("el-checkbox"), {
      "model-value": checkedCurrentAll.value && !disabledChecked.value,
      "indeterminate": !checkedCurrentAll.value && checkedCurrentSome.value,
      "onChange": onSelectAllChange,
      "disabled": disabledChecked.value
    }, null);
  }
  return [renderCell, renderTop];
}

export { useSelection };
//# sourceMappingURL=useSelection.mjs.map
